@@ -1,16 +1,15 @@
-#' Maximum likehood estimation for (SecSSE) with parameter as complex
-#' functions.
+#' Maximum likehood estimation for (SecSSE) complex functions as parameter
 #' 
 #' Maximum likehood estimation under Several examined and concealed
-#' States-dependent Speciation and Extinction (SecSSE) where some paramaters
+#' States-dependent Speciation and Extinction (SecSSE) where some parameters
 #' are functions of other parameters and/or factors.
 #' 
 #' @inheritParams default_params_doc
 #' 
-#' @return Parameter estimated and maximum likelihood
+#' @return Parameter estimates and maximum likelihood
 #' @examples
-#'# Example of how to set the arguments for a ML search.
-#'rm(list=ls(all=TRUE))
+#'# Example of how to set the arguments for an ML search. The ML search is stopped
+#'# after 10 iterations to keep run time short.
 #'library(secsse)
 #'library(DDD)
 #'set.seed(16)
@@ -59,12 +58,12 @@
 #'  par_6 <- par_3 * factor_1
 #'}
 #'
-#'tol = c(1e-02, 1e-03, 1e-04)
-#'maxiter = 1000 * round((1.25)^length(idparsopt))
-#'optimmethod = "subplex"
+#'tol = c(1e-03, 1e-03, 1e-03)
+#'optimmethod = "simplex"
 #'cond<-"proper_cond"
 #'root_state_weight <- "proper_weights"
 #'sampling_fraction <- c(1,1,1)
+#'maxiter <- 10
 #'model <- secsse_ml_func_def_pars(phylotree,
 #'traits,
 #'num_concealed_states,
@@ -84,7 +83,8 @@
 #'maxiter,
 #'optimmethod,
 #'num_cycles = 1)
-#'# ML -136.5796
+#'print(model$ML)
+#'# ML -136.45265 if run till completion
 #' @export
 secsse_ml_func_def_pars <- function(phy,
                                     traits,
@@ -104,10 +104,11 @@ secsse_ml_func_def_pars <- function(phy,
                                     tol = c(1E-4, 1E-5, 1E-7),
                                     maxiter = 1000 *
                                       round((1.25) ^ length(idparsopt)),
-                                    optimmethod = "subplex",
+                                    optimmethod = "simplex",
                                     num_cycles = 1,
                                     loglik_penalty = 0,
                                     is_complete_tree = FALSE,
+                                    take_into_account_root_edge = FALSE,
                                     num_threads = 1,
                                     atol = 1e-8,
                                     rtol = 1e-6,
@@ -133,6 +134,8 @@ secsse_ml_func_def_pars <- function(phy,
                    num_cycles = num_cycles,
                    loglik_penalty = loglik_penalty,
                    is_complete_tree = is_complete_tree,
+                   take_into_account_root_edge = 
+                     take_into_account_root_edge,
                    num_threads = num_threads,
                    atol = atol,
                    rtol = rtol,
@@ -151,8 +154,8 @@ secsse_ml_func_def_pars <- function(phy,
 #' 
 #' @return Parameter estimated and maximum likelihood
 #' @examples
-#'# Example of how to set the arguments for a ML search.
-#'rm(list=ls(all=TRUE))
+#'# Example of how to set the arguments for an ML search. The ML search is
+#'# stopped after 10 iterations to keep the run time short.
 #'library(secsse)
 #'library(DDD)
 #'set.seed(16)
@@ -204,11 +207,11 @@ secsse_ml_func_def_pars <- function(phy,
 #'}
 #'
 #'tol = c(1e-02, 1e-03, 1e-04)
-#'maxiter = 1000 * round((1.25)^length(idparsopt))
 #'optimmethod = 'subplex'
 #'cond <- 'proper_cond'
 #'root_state_weight <- 'proper_weights'
 #'sampling_fraction <- c(1,1,1)
+#'maxiter <- 10
 #'model <- cla_secsse_ml_func_def_pars(phylotree,
 #'traits,
 #'num_concealed_states,
@@ -228,7 +231,7 @@ secsse_ml_func_def_pars <- function(phy,
 #'maxiter,
 #'optimmethod,
 #'num_cycles = 1)
-#'# ML -136.5796
+#'# ML -136.5796 if run till completion
 #' @export
 cla_secsse_ml_func_def_pars <- function(phy,
                                         traits,
@@ -248,15 +251,17 @@ cla_secsse_ml_func_def_pars <- function(phy,
                                         tol = c(1e-04, 1e-05, 1e-07),
                                         maxiter = 1000 *
                                           round((1.25) ^ length(idparsopt)),
-                                        optimmethod = "subplex",
+                                        optimmethod = "simplex",
                                         num_cycles = 1,
                                         loglik_penalty = 0,
                                         is_complete_tree = FALSE,
-                                        verbose = (optimmethod == "simplex"),
+                                        take_into_account_root_edge = FALSE,
+                                        verbose = TRUE,
                                         num_threads = 1,
                                         atol = 1e-12,
                                         rtol = 1e-12,
-                                        method = "odeint::bulirsch_stoer") {
+                                        method = "odeint::bulirsch_stoer",
+                                        use_normalization = TRUE) {
   return(master_ml(phy = phy,
                    traits = traits,
                    num_concealed_states = num_concealed_states,
@@ -278,9 +283,11 @@ cla_secsse_ml_func_def_pars <- function(phy,
                    num_cycles = num_cycles,
                    loglik_penalty = loglik_penalty,
                    is_complete_tree = is_complete_tree,
+                   take_into_account_root_edge = take_into_account_root_edge,
                    verbose = verbose,
                    num_threads = num_threads,
                    atol = atol,
                    rtol = rtol,
-                   method = method))
+                   method = method,
+                   use_normalization = use_normalization))
 }
